==> enough_mail-0.0.34.tar.gz: lib/src/util/uint8_list_reader.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:enough_mail/src/util/ascii_runes.dart';

/// Combines several Uin8Lists to read from them sequentially
class Uint8ListReader {
  static const Utf8Decoder _utf8decoder = Utf8Decoder();
  Uint8List _data = Uint8List(0);

  void add(Uint8List list) {
    //idea: consider BytesBuilder
    if (_data.isEmpty) {
      _data = list;
    } else {
      _data = Uint8List.fromList(_data + list);
    }
  }

  void addText(String text) {
    add(Uint8List.fromList(text.codeUnits));
  }

  int findLineBreak() {
    var data = _data;
    for (var charIndex = 0; charIndex < data.length - 1; charIndex++) {
      if (data[charIndex] == 13 && data[charIndex + 1] == 10) {
        // ok found CR + LF sequence:
        return charIndex + 1;
      }
    }

    return null;
  }

  int findLastLineBreak() {
    var data = _data;
    for (var charIndex = data.length; --charIndex > 1;) {
      if (data[charIndex] == 10 && data[charIndex - 1] == 13) {
        // ok found CR + LF sequence:
        return charIndex;
      }
    }
    return null;
  }

  bool hasLineBreak() {
    return (findLineBreak() != null);
  }

  String readLine() {
    final pos = findLineBreak();
    if (pos == null) {
      return null;
    }
    final line = _utf8decoder.convert(_data, 0, pos - 1);
    _data = _data.sublist(pos + 1);
    return line;
  }

  List<String> readLines() {
    var pos = findLastLineBreak();
    if (pos == null) {
      return null;
    }
    String text;
    if (pos == _data.length - 1) {
      text = _utf8decoder.convert(_data);
      _data = Uint8List(0);
    } else {
      text = _utf8decoder.convert(_data, 0, pos);
      _data = _data.sublist(pos + 1);
    }
    return text.split('\r\n');
  }

  int findLastCrLfDotCrLfSequence() {
    var data = _data;
    for (var charIndex = data.length; --charIndex > 4;) {
      if (data[charIndex] == 10 &&
          data[charIndex - 1] == 13 &&
          data[charIndex - 2] == AsciiRunes.runeDot &&
          data[charIndex - 3] == 10 &&
          data[charIndex - 4] == 13) {
        // ok found CRLF.CRLF sequence:
        return charIndex;
      }
    }
    return null;
  }

  List<String> readLinesToCrLfDotCrLfSequence() {
    var pos = findLastCrLfDotCrLfSequence();
    if (pos == null) {
      return null;
    }
    String text;
    text = _utf8decoder.convert(_data, 0, pos - 4);
    if (pos == _data.length - 1) {
      _data = Uint8List(0);
    } else {
      _data = _data.sublist(pos + 1);
    }
    return text.split('\r\n');
  }

  Uint8List readBytes(int length) {
    if (!isAvailable(length)) {
      return null;
    }
    var result = _data.sublist(0, length);
    _data = _data.sublist(length);
    return result;
  }

  bool isAvailable(int length) {
    return (length <= _data.length);
  }
}

==> enough_mail-0.0.36.tar.gz: lib/src/util/uint8_list_reader.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:enough_mail/src/util/ascii_runes.dart';

/// Combines several Uin8Lists to read from them sequentially
class Uint8ListReader {
  static const Utf8Decoder _utf8decoder = Utf8Decoder(allowMalformed: true);
  Uint8List _data = Uint8List(0);

  void add(Uint8List list) {
    //idea: consider BytesBuilder
    if (_data.isEmpty) {
      _data = list;
    } else {
      _data = Uint8List.fromList(_data + list);
    }
  }

  void addText(String text) {
    add(Uint8List.fromList(text.codeUnits));
  }

  int findLineBreak() {
    var data = _data;
    for (var charIndex = 0; charIndex < data.length - 1; charIndex++) {
      if (data[charIndex] == 13 && data[charIndex + 1] == 10) {
        // ok found CR + LF sequence:
        return charIndex + 1;
      }
    }

    return null;
  }

  int findLastLineBreak() {
    var data = _data;
    for (var charIndex = data.length; --charIndex > 1;) {
      if (data[charIndex] == 10 && data[charIndex - 1] == 13) {
        // ok found CR + LF sequence:
        return charIndex;
      }
    }
    return null;
  }

  bool hasLineBreak() {
    return (findLineBreak() != null);
  }

  String readLine() {
    final pos = findLineBreak();
    if (pos == null) {
      return null;
    }
    final line = _utf8decoder.convert(_data, 0, pos - 1);
    _data = _data.sublist(pos + 1);
    return line;
  }

  List<String> readLines() {
    var pos = findLastLineBreak();
    if (pos == null) {
      return null;
    }
    String text;
    if (pos == _data.length - 1) {
      text = _utf8decoder.convert(_data);
      _data = Uint8List(0);
    } else {
      text = _utf8decoder.convert(_data, 0, pos);
      _data = _data.sublist(pos + 1);
    }
    return text.split('\r\n');
  }

  int findLastCrLfDotCrLfSequence() {
    var data = _data;
    for (var charIndex = data.length; --charIndex > 4;) {
      if (data[charIndex] == 10 &&
          data[charIndex - 1] == 13 &&
          data[charIndex - 2] == AsciiRunes.runeDot &&
          data[charIndex - 3] == 10 &&
          data[charIndex - 4] == 13) {
        // ok found CRLF.CRLF sequence:
        return charIndex;
      }
    }
    return null;
  }

  List<String> readLinesToCrLfDotCrLfSequence() {
    var pos = findLastCrLfDotCrLfSequence();
    if (pos == null) {
      return null;
    }
    String text;
    text = _utf8decoder.convert(_data, 0, pos - 4);
    if (pos == _data.length - 1) {
      _data = Uint8List(0);
    } else {
      _data = _data.sublist(pos + 1);
    }
    return text.split('\r\n');
  }

  Uint8List readBytes(int length) {
    if (!isAvailable(length)) {
      return null;
    }
    var result = _data.sublist(0, length);
    _data = _data.sublist(length);
    return result;
  }

  bool isAvailable(int length) {
    return (length <= _data.length);
  }
}
